'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var DEFAULT_COLOR = 'red';
var DEFAULT_BORDER = '1px solid ' + DEFAULT_COLOR;

var BASE_STYLES = {
  range: {
    start: {
      color: DEFAULT_COLOR,
      borderBottom: DEFAULT_BORDER,
      borderLeft: DEFAULT_BORDER,
      borderTop: DEFAULT_BORDER
    },
    between: {
      color: DEFAULT_COLOR,
      borderBottom: DEFAULT_BORDER,
      borderTop: DEFAULT_BORDER
    },
    end: {
      color: DEFAULT_COLOR,
      borderBottom: DEFAULT_BORDER,
      borderRight: DEFAULT_BORDER,
      borderTop: DEFAULT_BORDER
    }
  },
  highlight: {
    color: DEFAULT_COLOR,
    borderBottom: DEFAULT_BORDER
  }
};

exports['default'] = _react2['default'].createClass({
  displayName: 'TextHighlighting',

  propTypes: {
    highlightChars: _react2['default'].PropTypes.array,
    highlightCharsRanges: _react2['default'].PropTypes.array,
    highlight: _react2['default'].PropTypes.string,
    styles: _react2['default'].PropTypes.object
  },

  getInitialState: function getInitialState() {
    var highlightRanges = this.props.highlight ? this.getRangesFromString(this.props.highlight) : null;

    return {
      chars: this.props.children.split(''),
      highlightRanges: highlightRanges
    };
  },

  getRangesFromString: function getRangesFromString(string) {
    var regex = new RegExp(string, 'gi');

    var result = undefined;
    var indices = [];

    while (result = regex.exec(this.props.children)) {
      indices.push(result.index);
    }

    return indices.map(function (indice) {
      return { start: indice, end: indice + string.length - 1 };
    });
  },

  getStyles: function getStyles() {
    return this.props.styles || BASE_STYLES;
  },

  getStylesForRanges: function getStylesForRanges(ranges, index) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = ranges[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var range = _step.value;

        var styles = this.getStylesForRange(range, index);
        if (styles) return styles;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator['return']) {
          _iterator['return']();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  },

  getStylesForRange: function getStylesForRange(range, index) {
    var styles = this.getStyles();

    if (range.start === range.end && range.end === index) {
      return Object.assign(styles.range.start, styles.range.end);
    }

    if (range.start === index) {
      return styles.range.start;
    }

    if (range.end === index) {
      return styles.range.end;
    }

    if (range.start < index && range.end > index) {
      return styles.range.between;
    }
  },

  render: function render() {
    var _this = this;

    var coloredChars = this.state.chars.map(function (char, index) {
      var style = {};

      if (_this.props.highlightChars && _this.props.highlightChars.includes(index)) {
        style = _this.getStyles().highlight;
      }

      if (_this.state.highlightRanges) {
        style = _this.getStylesForRanges(_this.state.highlightRanges, index);
      }

      if (_this.props.highlightCharsRanges) {
        style = _this.getStylesForRanges(_this.props.highlightCharsRanges, index);
      }

      return _react2['default'].createElement(
        'span',
        { key: index, style: style },
        char
      );
    });

    return _react2['default'].createElement(
      'span',
      this.props,
      coloredChars
    );
  }
});
module.exports = exports['default'];